package com.android.basiclib.engine.dialog

import android.view.LayoutInflater
import android.view.View
import androidx.fragment.app.FragmentActivity
import androidx.viewbinding.ViewBinding
import com.android.basiclib.engine.dialog.callback.IPopupController
import com.android.basiclib.engine.dialog.creator.PopupViewCreatorFactory
import com.lxj.xpopup.XPopup
import com.lxj.xpopup.enums.PopupAnimation

/**
 * 弹窗的弹出引擎，目前使用Xpopup
 */
fun <VB : ViewBinding> FragmentActivity.showPopup(
    popupType: PopupType,  //必须指定类型
    viewBinding: (LayoutInflater) -> VB,
    popupAnimationType: PopupAnimationType? = null,  //动画类型
    targetView: View? = null, //如果是Attach则一定要指定在哪个View的基础上弹窗
    width: Int = 0,
    height: Int = 0,   //宽高一般不需要额外设置，自动适配布局的宽高
    maxWidth: Int = 0,  //如果想要指定宽高，可以自定义宽高
    maxHeight: Int = 0,
    offsetX: Int = 0,   //弹框在 X,Y 方向的偏移值
    offsetY: Int = 0,
    enableDismiss: Boolean = true,
    isCenterHorizontal: Boolean = false,  //布局是否水平居中
    onCreateListener: ((VB, IPopupController) -> Unit)? = null,   //创建的回调
    onDismissListener: (() -> Unit)? = null,     //消失的回调
) {

    //根据类型转换对应的PopupView继承
    val creator =
        PopupViewCreatorFactory.getCreator<VB>(popupType, this, width, height, maxWidth, maxHeight, onCreateListener, onDismissListener)
    val popupView = creator.create(viewBinding)

    val builder = XPopup.Builder(this)
        .hasShadowBg(popupType == PopupType.BOTTOM || popupType == PopupType.CENTER)  //默认底部弹窗和中间弹窗展示灰色背景遮罩
        .isDestroyOnDismiss(true)  //消失的时候是否释放资源
        .offsetX(offsetX)
        .offsetY(offsetY)
        .isCenterHorizontal(isCenterHorizontal)

    if (targetView != null) {
        builder.atView(targetView)
    }

    if (popupType == PopupType.KEYBOARD) {
        builder.autoOpenSoftInput(true)
            .moveUpToKeyboard(true)
    }

    if (!enableDismiss) {
        builder.dismissOnBackPressed(false)
            .dismissOnTouchOutside(false)
    }

    //转换动画效果，是否需要添加动画
    if (transformAnimationType(popupAnimationType) != null) {
        builder.popupAnimation(transformAnimationType(popupAnimationType))
    }

    builder.asCustom(popupView)
        .show()
}

//动画转换
fun transformAnimationType(animationType: PopupAnimationType?): PopupAnimation? {

    return when (animationType) {
        PopupAnimationType.ScaleAlphaFromCenter -> PopupAnimation.ScaleAlphaFromCenter
        PopupAnimationType.ScaleAlphaFromLeftTop -> PopupAnimation.ScaleAlphaFromLeftTop
        PopupAnimationType.ScaleAlphaFromRightTop -> PopupAnimation.ScaleAlphaFromRightTop
        PopupAnimationType.ScaleAlphaFromLeftBottom -> PopupAnimation.ScaleAlphaFromLeftBottom
        PopupAnimationType.ScaleAlphaFromRightBottom -> PopupAnimation.ScaleAlphaFromRightBottom
        PopupAnimationType.TranslateAlphaFromLeft -> PopupAnimation.TranslateAlphaFromLeft
        PopupAnimationType.TranslateAlphaFromRight -> PopupAnimation.TranslateAlphaFromRight
        PopupAnimationType.TranslateAlphaFromTop -> PopupAnimation.TranslateAlphaFromTop
        PopupAnimationType.TranslateAlphaFromBottom -> PopupAnimation.TranslateAlphaFromBottom
        PopupAnimationType.TranslateFromLeft -> PopupAnimation.TranslateFromLeft
        PopupAnimationType.TranslateFromRight -> PopupAnimation.TranslateFromRight
        PopupAnimationType.TranslateFromTop -> PopupAnimation.TranslateFromTop
        PopupAnimationType.TranslateFromBottom -> PopupAnimation.TranslateFromBottom
        PopupAnimationType.NoAnimation -> PopupAnimation.NoAnimation
        else -> null
    }

}


