package com.android.basiclib.engine.image_select

import android.app.Activity
import android.content.Context
import android.content.ContextWrapper
import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import android.net.Uri
import android.widget.ImageView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.android.basiclib.utils.CommUtils
import com.bumptech.glide.Glide
import com.bumptech.glide.request.target.CustomTarget
import com.bumptech.glide.request.transition.Transition
import com.luck.picture.lib.R
import com.luck.picture.lib.engine.CropFileEngine
import com.luck.picture.lib.utils.StyleUtils
import com.yalantis.ucrop.UCrop
import com.yalantis.ucrop.UCropImageEngine

/**
 * 图片裁剪引擎，可用于 ImageSelectEngine 中设置，实现 CropFileEngine 接口可以在 ImageSelectEngine 中自动配置相机和相册后的裁剪
 *
 *
 * 也可以单独使用，设置本地图片的Uri和目标Path的Uri，即可单独实现裁剪
 */
class ImageFileCropEngine(private val ratioX: Int, private val ratioY: Int) : CropFileEngine {

    override fun onStartCrop(fragment: Fragment, srcUri: Uri, destinationUri: Uri, dataSource: ArrayList<String>, requestCode: Int) {
        val options = buildOptions(CommUtils.getContext(), ratioX, ratioY)
        val uCrop: UCrop = if (dataSource.isEmpty()) {
            UCrop.of<Any>(srcUri, destinationUri)
        } else {
            UCrop.of(srcUri, destinationUri, dataSource)
        }
        uCrop.withOptions(options)
        uCrop.setImageEngine(object : UCropImageEngine {
            override fun loadImage(context: Context, url: String, imageView: ImageView) {
                if (!assertValidRequest(context)) {
                    return
                }
                Glide.with(context).load(url).override(180, 180).into(imageView)
            }

            override fun loadImage(context: Context, url: Uri, maxWidth: Int, maxHeight: Int, call: UCropImageEngine.OnCallbackListener<Bitmap>) {
                Glide.with(context).asBitmap().load(url).override(maxWidth, maxHeight).into(object : CustomTarget<Bitmap?>() {
                    override fun onResourceReady(resource: Bitmap, transition: Transition<in Bitmap?>?) {
                        call.onCall(resource)
                    }

                    override fun onLoadCleared(placeholder: Drawable?) {
                        call.onCall(null)
                    }
                })
            }
        })
        uCrop.start(fragment.requireActivity(), fragment, requestCode)
    }

    /**
     * 配制UCrop，可根据需求自我扩展
     */
    private fun buildOptions(context: Context, ratioX: Int, ratioY: Int): UCrop.Options {
        val options = UCrop.Options()
//        options.setHideBottomControls(!cb_hide.isChecked());
//        options.setFreeStyleCropEnabled(cb_styleCrop.isChecked());
//        options.setShowCropFrame(cb_showCropFrame.isChecked());
//        options.setShowCropGrid(cb_showCropGrid.isChecked());
//        options.setCircleDimmedLayer(cb_crop_circular.isChecked());
//        options.setCropOutputPathDir(getSandboxPath());
//        options.setSkipCropMimeType(getNotSupportCrop());
//        options.isForbidCropGifWebp(cb_not_gif.isChecked());
        options.withAspectRatio(ratioX.toFloat(), ratioY.toFloat()) //裁剪比例
        options.isCropDragSmoothToCenter(false)
        options.isForbidSkipMultipleCrop(true)
        options.setMaxScaleMultiplier(100f)
        val selectorStyle = ImageSelectEngine.selectorStyle
        if (selectorStyle.selectMainStyle.statusBarColor != 0) {
            val mainStyle = selectorStyle.selectMainStyle
            val isDarkStatusBarBlack = mainStyle.isDarkStatusBarBlack
            val statusBarColor = mainStyle.statusBarColor
            options.isDarkStatusBarBlack(isDarkStatusBarBlack)
            if (StyleUtils.checkStyleValidity(statusBarColor)) {
                options.setStatusBarColor(statusBarColor)
                options.setToolbarColor(statusBarColor)
            } else {
                options.setStatusBarColor(ContextCompat.getColor(context, R.color.ps_color_grey))
                options.setToolbarColor(ContextCompat.getColor(context, R.color.ps_color_grey))
            }
            val titleBarStyle = selectorStyle.titleBarStyle
            if (StyleUtils.checkStyleValidity(titleBarStyle.titleTextColor)) {
                options.setToolbarWidgetColor(titleBarStyle.titleTextColor)
            } else {
                options.setToolbarWidgetColor(ContextCompat.getColor(context, R.color.ps_color_white))
            }
        } else {
            options.setStatusBarColor(ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_grey))
            options.setToolbarColor(ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_grey))
            options.setToolbarWidgetColor(ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_white))
        }
        return options
    }



    fun assertValidRequest(context: Context?): Boolean {
        if (context is Activity) {
            return !isDestroy(context)
        } else if (context is ContextWrapper) {
            if (context.baseContext is Activity) {
                val activity = context.baseContext as Activity
                return !isDestroy(activity)
            }
        }
        return true
    }

    private fun isDestroy(activity: Activity?): Boolean {
        return if (activity == null) {
            true
        } else activity.isFinishing || activity.isDestroyed
    }

}
