package com.android.basiclib.engine.image_select

import android.Manifest
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.app.Activity
import android.content.Context
import android.content.pm.ActivityInfo
import android.view.animation.AnimationUtils
import androidx.core.content.ContextCompat
import com.android.basiclib.engine.image_select.result.ImageSelectEntity
import com.android.basiclib.engine.image_select.result.OnImageSelectResult
import com.android.basiclib.engine.permission.requestPermission
import com.android.basiclib.engine.toast.toast
import com.android.basiclib.utils.CheckUtil
import com.android.basiclib.utils.CommUtils
import com.luck.picture.lib.R
import com.luck.picture.lib.animators.AnimationType
import com.luck.picture.lib.basic.PictureSelector
import com.luck.picture.lib.config.PictureMimeType
import com.luck.picture.lib.config.SelectMimeType
import com.luck.picture.lib.engine.UriToFileTransformEngine
import com.luck.picture.lib.entity.LocalMedia
import com.luck.picture.lib.interfaces.OnKeyValueResultCallbackListener
import com.luck.picture.lib.interfaces.OnQueryFilterListener
import com.luck.picture.lib.interfaces.OnResultCallbackListener
import com.luck.picture.lib.style.BottomNavBarStyle
import com.luck.picture.lib.style.PictureSelectorStyle
import com.luck.picture.lib.style.SelectMainStyle
import com.luck.picture.lib.style.TitleBarStyle
import com.luck.picture.lib.utils.SandboxTransformUtils

object ImageSelectEngine {

    @JvmField
    var selectorStyle: PictureSelectorStyle = PictureSelectorStyle()

    init {
        // 主体风格
        val numberSelectMainStyle = SelectMainStyle()
        numberSelectMainStyle.isSelectNumberStyle = true
        numberSelectMainStyle.isPreviewSelectNumberStyle = false
        numberSelectMainStyle.isPreviewDisplaySelectGallery = true
        numberSelectMainStyle.selectBackground = R.drawable.ps_default_num_selector
        numberSelectMainStyle.previewSelectBackground = R.drawable.ps_preview_checkbox_selector
        numberSelectMainStyle.selectNormalBackgroundResources = R.drawable.ps_select_complete_normal_bg
        numberSelectMainStyle.selectNormalTextColor = ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_53575e)
        numberSelectMainStyle.setSelectNormalText(R.string.ps_send)
        numberSelectMainStyle.adapterPreviewGalleryBackgroundResource = R.drawable.ps_preview_gallery_bg
        numberSelectMainStyle.adapterPreviewGalleryItemSize = CommUtils.dip2px(52)
        numberSelectMainStyle.setPreviewSelectText(R.string.ps_select)
        numberSelectMainStyle.previewSelectTextSize = 14
        numberSelectMainStyle.previewSelectTextColor = ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_white)
        numberSelectMainStyle.previewSelectMarginRight = CommUtils.dip2px(6)
        numberSelectMainStyle.selectBackgroundResources = R.drawable.ps_select_complete_bg
        numberSelectMainStyle.setSelectText(R.string.ps_send_num)
        numberSelectMainStyle.selectTextColor = ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_white)
        numberSelectMainStyle.mainListBackgroundColor = ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_black)
        numberSelectMainStyle.isCompleteSelectRelativeTop = true
        numberSelectMainStyle.isPreviewSelectRelativeBottom = true
        numberSelectMainStyle.isAdapterItemIncludeEdge = false

        // 头部TitleBar 风格
        val numberTitleBarStyle = TitleBarStyle()
        numberTitleBarStyle.isHideCancelButton = true
        numberTitleBarStyle.isAlbumTitleRelativeLeft = true
        numberTitleBarStyle.titleAlbumBackgroundResource = R.drawable.ps_album_bg
        numberTitleBarStyle.titleDrawableRightResource = R.drawable.ps_ic_grey_arrow
        numberTitleBarStyle.previewTitleLeftBackResource = R.drawable.ps_ic_normal_back

        // 底部NavBar 风格
        val numberBottomNavBarStyle = BottomNavBarStyle()
        numberBottomNavBarStyle.bottomPreviewNarBarBackgroundColor =
            ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_half_grey)
        numberBottomNavBarStyle.setBottomPreviewNormalText(R.string.ps_preview)
        numberBottomNavBarStyle.bottomPreviewNormalTextColor = ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_9b)
        numberBottomNavBarStyle.bottomPreviewNormalTextSize = 16
        numberBottomNavBarStyle.isCompleteCountTips = false
        numberBottomNavBarStyle.setBottomPreviewSelectText(R.string.ps_preview_num)
        numberBottomNavBarStyle.bottomPreviewSelectTextColor = ContextCompat.getColor(CommUtils.getContext(), R.color.ps_color_white)
        selectorStyle.titleBarStyle = numberTitleBarStyle
        selectorStyle.bottomBarStyle = numberBottomNavBarStyle
        selectorStyle.selectMainStyle = numberSelectMainStyle
    }

    /**
     * 自定义沙盒文件处理
     */
    private class MeSandboxFileEngine : UriToFileTransformEngine {
        override fun onUriToFileAsyncTransform(
            context: Context,
            srcPath: String,
            mineType: String,
            call: OnKeyValueResultCallbackListener
        ) {
            call.onCallback(srcPath, SandboxTransformUtils.copyPathToSandbox(context, srcPath, mineType))
        }
    }

    private val notSupportCrop: Array<String>
        //不支持裁剪的格式
        get() = arrayOf(PictureMimeType.ofGIF(), PictureMimeType.ofWEBP())

    /**
     * 打开相机
     */
    fun openCamera(
        activity: Activity,
        selectedList: List<LocalMedia>?,
        canCrop: Boolean = false,
        canCompress: Boolean = true,
        ratioX: Int = 1,
        ratioY: Int = 1,
        resultListener: OnImageSelectResult?
    ) {

        activity.requestPermission(
            Manifest.permission.CAMERA,
            Manifest.permission.READ_MEDIA_IMAGES,  //Target33之后Android7的手机也需要申请此权限
            Manifest.permission.WRITE_EXTERNAL_STORAGE
        ) {
            PictureSelector.create(activity)
                .openCamera(SelectMimeType.ofImage())
                .setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED) // 设置相册Activity方向，不设置默认使用系统
                .setCropEngine(if (canCrop) ImageFileCropEngine(ratioX, ratioY) else null)
                .setCompressEngine(if (canCompress) ImageFileCompressEngine() else null)
                .setSandboxFileEngine(MeSandboxFileEngine())
                .setSelectedData(selectedList)
                .forResult(ImageSelectResultListener(resultListener))
        }
    }

    //自定义的数据回调
    class ImageSelectResultListener(private val listener: OnImageSelectResult?) :
        OnResultCallbackListener<LocalMedia?> {
        override fun onResult(result: java.util.ArrayList<LocalMedia?>?) {
            val dataList = ArrayList<ImageSelectEntity>()

            result?.let {
                for (item in it) {
                    item?.let { localMedia ->
                        val compressPath = localMedia.compressPath
                        val realPath = localMedia.realPath
                        val path = if (CheckUtil.isEmpty(compressPath)) realPath else compressPath

                        dataList.add(
                            ImageSelectEntity(
                                localMedia.mimeType,
                                path,
                                localMedia.videoThumbnailPath,
                                localMedia.duration
                            )
                        )
                    }
                }
            }

            listener?.onResult(dataList)
        }

        override fun onCancel() {
            toast(R.string.ps_cancel)
        }
    }


    /**
     * 选择相册
     */
    fun openAlbum(
        activity: Activity,
        selectedList: List<LocalMedia>?,
        selectNum: Int,
        canTackPhoto: Boolean = false,
        canVideo: Boolean = false,
        canCrop: Boolean = false,
        canCompress: Boolean = true,
        ratioX: Int = 1,
        ratioY: Int = 1,
        resultListener: OnImageSelectResult?
    ) {
        val permissionList: MutableList<String> = ArrayList()
        permissionList.add(Manifest.permission.READ_MEDIA_IMAGES)
        permissionList.add(Manifest.permission.WRITE_EXTERNAL_STORAGE)
        if (canVideo) {
            permissionList.add(Manifest.permission.READ_MEDIA_VIDEO)
        }
        if (canTackPhoto) {
            permissionList.add(Manifest.permission.CAMERA)
        }
        val permissionArray = permissionList.toTypedArray<String>()

        activity.requestPermission(*permissionArray) {
            // 正式开启相册
            PictureSelector.create(activity)
                .openGallery(if (canVideo) SelectMimeType.ofAll() else SelectMimeType.ofImage())
                .setSelectorUIStyle(selectorStyle)
                .setImageEngine(ImageSelectGlideEngine) // 外部传入图片加载引擎，必传项
                .setCropEngine(if (canCrop) ImageFileCropEngine(ratioX, ratioY) else null)
                .setCompressEngine(if (canCompress) ImageFileCompressEngine() else null)
                .setSandboxFileEngine(MeSandboxFileEngine())
                .setSelectedData(selectedList)
                .isLoopAutoVideoPlay(true)
                .isUseSystemVideoPlayer(true)
                .isPageSyncAlbumCount(true)
                .setQueryFilterListener(OnQueryFilterListener { return@OnQueryFilterListener false })
                .setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED) // 设置相册Activity方向，不设置默认使用系统
                .isOriginalControl(false)
                .isDisplayCamera(canTackPhoto)
                .isOpenClickSound(false)
                .setSkipCropMimeType(*notSupportCrop)
                .isFastSlidingSelect(false)
                .isWithSelectVideoImage(true)
                .isPreviewFullScreenMode(false)
                .isVideoPauseResumePlay(true)
                .isPreviewZoomEffect(true)
                .isPreviewImage(true)
                .isPreviewVideo(true)
                .isPreviewAudio(true)
                .setGridItemSelectAnimListener { view, isSelected ->
                    val set = AnimatorSet()
                    set.playTogether(
                        ObjectAnimator.ofFloat(view, "scaleX", if (isSelected) 1f else 1.12f, if (isSelected) 1.12f else 1.0f),
                        ObjectAnimator.ofFloat(view, "scaleY", if (isSelected) 1f else 1.12f, if (isSelected) 1.12f else 1.0f)
                    )
                    set.setDuration(350)
                    set.start()
                }
                .setSelectAnimListener { view ->
                    val animation = AnimationUtils.loadAnimation(CommUtils.getContext(), R.anim.ps_anim_modal_in)
                    view.startAnimation(animation)
                    return@setSelectAnimListener animation.duration
                }
                .isMaxSelectEnabledMask(true)
                .isDirectReturnSingle(true)
                .setMaxSelectNum(selectNum)
                .setRecyclerAnimationMode(AnimationType.DEFAULT_ANIMATION)
                .isGif(true)
                .forResult(ImageSelectResultListener(resultListener))
        }

    }

    private fun cleanAllCache() {}

}
