package com.android.basiclib.engine.preferences

import android.app.Application
import android.content.Context
import androidx.datastore.core.DataStore
import androidx.datastore.preferences.core.*
import androidx.datastore.preferences.preferencesDataStore
import com.android.basiclib.utils.result.ISAFLauncher
import kotlin.properties.ReadOnlyProperty
import kotlin.reflect.KProperty

/**
接口的默认实现类，如果想要多个Store分区，可以指定对应的分区文件name ，例如：

//可以继承实现
object AuthRepository : DataStoreOwner(name = "auth") {
val counter by intPreference()
val language by stringPreference(default = "zh")
}

//也可以委托实现
object AuthRepository : BaseRepository(), IDataStoreOwner by DataStoreOwner(name = "auth") {
val counter by intPreference()
val language by stringPreference(default = "zh")
}

 默认为全局一个分区文件，如果不想要多分区，那么不要直接使用 DataStoreOwner 可以用 EasyDataStore 来统一管理快速实现
 */
open class DataStoreOwner(name: String = IDataStoreOwner.application.packageName) : IDataStoreOwner {
    private val Context.dataStore by preferencesDataStore(name)  //通过内置扩展初始化，赋值给扩展属性
    override val dataStore get() = context.dataStore  //扩展属性再赋值给 dataStore 的变量
}

//别名
typealias store = IDataStoreOwner

// 定义接口，方便Kotlin委托实现
interface IDataStoreOwner {
    val context: Context get() = application

    val dataStore: DataStore<Preferences>  //空实现

    fun intPreference(default: Int? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<Int>> =
        PreferenceProperty(::intPreferencesKey, default)

    fun doublePreference(default: Double? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<Double>> =
        PreferenceProperty(::doublePreferencesKey, default)

    fun longPreference(default: Long? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<Long>> =
        PreferenceProperty(::longPreferencesKey, default)

    fun floatPreference(default: Float? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<Float>> =
        PreferenceProperty(::floatPreferencesKey, default)

    fun booleanPreference(default: Boolean? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<Boolean>> =
        PreferenceProperty(::booleanPreferencesKey, default)

    fun stringPreference(default: String? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<String>> =
        PreferenceProperty(::stringPreferencesKey, default)

    fun stringSetPreference(default: Set<String>? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<Set<String>>> =
        PreferenceProperty(::stringSetPreferencesKey, default)

    fun byteArrayPreference(default: ByteArray? = null): ReadOnlyProperty<IDataStoreOwner, DataStorePreference<ByteArray>> =
        PreferenceProperty(::byteArrayPreferencesKey, default)

    //自定义类，封装 KV 对象，内部使用 DataStorePreference 具体操作 KV 的存取
    class PreferenceProperty<V>(
        private val key: (String) -> Preferences.Key<V>,
        private val default: V? = null,
    ) : ReadOnlyProperty<IDataStoreOwner, DataStorePreference<V>> {
        //做缓存，如果已经取出了，不需要重复取
        private var cache: DataStorePreference<V>? = null

        override fun getValue(owner: IDataStoreOwner, property: KProperty<*>): DataStorePreference<V> =
            cache ?: DataStorePreference(owner.dataStore, key(property.name), default).also { cache = it }
    }

    companion object {
        internal lateinit var application: Application
    }
}